@@ -2,7 +2,7 @@
 // buffer.cpp
 // ~~~~~~~~~~
 //
-// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -71,6 +71,25 @@ public:
   const_iterator end() const { return 0; }
 };
 
+template <typename T, std::size_t Extent>
+struct span
+{
+  T* data() const { return 0; }
+  std::size_t size() const { return 0; }
+  friend span<const T, Extent> as_bytes(const span&) { return {}; }
+  span<T, static_cast<std::size_t>(-1)>
+    subspan(std::size_t, std::size_t = 0) const { return {}; }
+};
+
+template <typename T, std::size_t Extent>
+struct span<const T, Extent>
+{
+  T* data() const { return 0; }
+  std::size_t size() const { return 0; }
+  span<const T, static_cast<std::size_t>(-1)>
+    subspan(std::size_t, std::size_t = 0) const { return {}; }
+};
+
 void test()
 {
   try
@@ -79,6 +98,8 @@ void test()
     const char const_raw_data[1024] = "";
     void* void_ptr_data = raw_data;
     const void* const_void_ptr_data = const_raw_data;
+    span<char, 1> span_1;
+    span<const char, 1> span_2;
 #if defined(BOOST_ASIO_HAS_BOOST_ARRAY)
     boost::array<char, 1024> array_data;
     const boost::array<char, 1024>& const_array_data_1 = array_data;
@@ -107,8 +128,10 @@ void test()
 
     mutable_buffer mb1;
     mutable_buffer mb2(void_ptr_data, 1024);
-    mutable_buffer mb3(mb1);
+    mutable_buffer mb3(span_1);
     (void)mb3;
+    mutable_buffer mb4(mb1);
+    (void)mb4;
 
     // mutable_buffer functions.
 
@@ -124,30 +147,18 @@ void test()
     mb1 = mb2 + 128;
     mb1 = 128 + mb2;
 
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-
-    // mutable_buffers_1 constructors.
-
-    mutable_buffers_1 mbc1(mb1);
-    mutable_buffers_1 mbc2(mbc1);
-
-    // mutable_buffers_1 functions.
-
-    mutable_buffers_1::const_iterator iter1 = mbc1.begin();
-    (void)iter1;
-    mutable_buffers_1::const_iterator iter2 = mbc1.end();
-    (void)iter2;
-
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
     // const_buffer constructors.
 
     const_buffer cb1;
     const_buffer cb2(const_void_ptr_data, 1024);
-    const_buffer cb3(cb1);
+    const_buffer cb3(span_1);
     (void)cb3;
-    const_buffer cb4(mb1);
+    const_buffer cb4(span_2);
     (void)cb4;
+    const_buffer cb5(cb1);
+    (void)cb5;
+    const_buffer cb6(mb1);
+    (void)cb6;
 
     // const_buffer functions.
 
@@ -163,47 +174,16 @@ void test()
     cb1 = cb2 + 128;
     cb1 = 128 + cb2;
 
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-
-    // const_buffers_1 constructors.
-
-    const_buffers_1 cbc1(cb1);
-    const_buffers_1 cbc2(cbc1);
-
-    // const_buffers_1 functions.
-
-    const_buffers_1::const_iterator iter3 = cbc1.begin();
-    (void)iter3;
-    const_buffers_1::const_iterator iter4 = cbc1.end();
-    (void)iter4;
-
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
     // buffer_size function overloads.
 
     std::size_t size1 = buffer_size(mb1);
     (void)size1;
     std::size_t size2 = buffer_size(cb1);
     (void)size2;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size3 = buffer_size(mbc1);
+    std::size_t size3 = buffer_size(mutable_buffer_sequence);
     (void)size3;
-    std::size_t size4 = buffer_size(cbc1);
+    std::size_t size4 = buffer_size(const_buffer_sequence);
     (void)size4;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size5 = buffer_size(mutable_buffer_sequence);
-    (void)size5;
-    std::size_t size6 = buffer_size(const_buffer_sequence);
-    (void)size6;
-
-    // buffer_cast function overloads.
-
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    void* ptr3 = buffer_cast<void*>(mb1);
-    (void)ptr3;
-    const void* ptr4 = buffer_cast<const void*>(cb1);
-    (void)ptr4;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
     // buffer function overloads.
 
@@ -254,88 +234,32 @@ void test()
 
     // buffer_copy function overloads.
 
-    std::size_t size7 = buffer_copy(mb1, cb2);
+    std::size_t size5 = buffer_copy(mb1, cb2);
+    (void)size5;
+    std::size_t size6 = buffer_copy(mb1, mb2);
+    (void)size6;
+    std::size_t size7 = buffer_copy(mb1, const_buffer_sequence);
     (void)size7;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size8 = buffer_copy(mb1, cbc2);
+    std::size_t size8 = buffer_copy(mutable_buffer_sequence, cb2);
     (void)size8;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size9 = buffer_copy(mb1, mb2);
+    std::size_t size9 = buffer_copy(mutable_buffer_sequence, mb2);
     (void)size9;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size10 = buffer_copy(mb1, mbc2);
-    (void)size10;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size11 = buffer_copy(mb1, const_buffer_sequence);
-    (void)size11;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size12 = buffer_copy(mbc1, cb2);
-    (void)size12;
-    std::size_t size13 = buffer_copy(mbc1, cbc2);
-    (void)size13;
-    std::size_t size14 = buffer_copy(mbc1, mb2);
-    (void)size14;
-    std::size_t size15 = buffer_copy(mbc1, mbc2);
-    (void)size15;
-    std::size_t size16 = buffer_copy(mbc1, const_buffer_sequence);
-    (void)size16;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size17 = buffer_copy(mutable_buffer_sequence, cb2);
-    (void)size17;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size18 = buffer_copy(mutable_buffer_sequence, cbc2);
-    (void)size18;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size19 = buffer_copy(mutable_buffer_sequence, mb2);
-    (void)size19;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size20 = buffer_copy(mutable_buffer_sequence, mbc2);
-    (void)size20;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size21 = buffer_copy(
+    std::size_t size10 = buffer_copy(
         mutable_buffer_sequence, const_buffer_sequence);
-    (void)size21;
-    std::size_t size22 = buffer_copy(mb1, cb2, 128);
-    (void)size22;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size23 = buffer_copy(mb1, cbc2, 128);
-    (void)size23;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size24 = buffer_copy(mb1, mb2, 128);
-    (void)size24;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size25 = buffer_copy(mb1, mbc2, 128);
-    (void)size25;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size26 = buffer_copy(mb1, const_buffer_sequence, 128);
-    (void)size26;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size27 = buffer_copy(mbc1, cb2, 128);
-    (void)size27;
-    std::size_t size28 = buffer_copy(mbc1, cbc2, 128);
-    (void)size28;
-    std::size_t size29 = buffer_copy(mbc1, mb2, 128);
-    (void)size29;
-    std::size_t size30 = buffer_copy(mbc1, mbc2, 128);
-    (void)size30;
-    std::size_t size31 = buffer_copy(mbc1, const_buffer_sequence, 128);
-    (void)size31;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size32 = buffer_copy(mutable_buffer_sequence, cb2, 128);
-    (void)size32;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size33 = buffer_copy(mutable_buffer_sequence, cbc2, 128);
-    (void)size33;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size34 = buffer_copy(mutable_buffer_sequence, mb2, 128);
-    (void)size34;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size35 = buffer_copy(mutable_buffer_sequence, mbc2, 128);
-    (void)size35;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    std::size_t size36 = buffer_copy(
+    (void)size10;
+    std::size_t size11 = buffer_copy(mb1, cb2, 128);
+    (void)size11;
+    std::size_t size12 = buffer_copy(mb1, mb2, 128);
+    (void)size12;
+    std::size_t size13 = buffer_copy(mb1, const_buffer_sequence, 128);
+    (void)size13;
+    std::size_t size14 = buffer_copy(mutable_buffer_sequence, cb2, 128);
+    (void)size14;
+    std::size_t size15 = buffer_copy(mutable_buffer_sequence, mb2, 128);
+    (void)size15;
+    std::size_t size16 = buffer_copy(
         mutable_buffer_sequence, const_buffer_sequence, 128);
-    (void)size36;
+    (void)size16;
 
     // dynamic_buffer function overloads.
 
@@ -354,24 +278,24 @@ void test()
 
     // dynamic_buffer member functions.
 
-    std::size_t size37 = db1.size();
-    (void)size37;
-    std::size_t size38 = db3.size();
-    (void)size38;
+    std::size_t size17 = db1.size();
+    (void)size17;
+    std::size_t size18 = db3.size();
+    (void)size18;
 
-    std::size_t size39 = db1.max_size();
-    (void)size39;
-    std::size_t size40 = db3.max_size();
-    (void)size40;
+    std::size_t size19 = db1.max_size();
+    (void)size19;
+    std::size_t size20 = db3.max_size();
+    (void)size20;
 
 #if !defined(BOOST_ASIO_NO_DYNAMIC_BUFFER_V1)
     dynamic_string_buffer<char, std::string::traits_type,
       std::string::allocator_type>::const_buffers_type
-        cb5 = db1.data();
-    (void)cb5;
+        cb7 = db1.data();
+    (void)cb7;
     dynamic_vector_buffer<char, std::allocator<char> >::const_buffers_type
-      cb6 = db3.data();
-    (void)cb6;
+      cb8 = db3.data();
+    (void)cb8;
 
     dynamic_string_buffer<char, std::string::traits_type,
       std::string::allocator_type>::mutable_buffers_type mb5
@@ -395,14 +319,14 @@ void test()
 
     dynamic_string_buffer<char, std::string::traits_type,
       std::string::allocator_type>::const_buffers_type
-        cb7 = static_cast<const dynamic_string_buffer<char,
+        cb9 = static_cast<const dynamic_string_buffer<char,
           std::string::traits_type,
             std::string::allocator_type>&>(db1).data(0, 1);
-    (void)cb7;
+    (void)cb9;
     dynamic_vector_buffer<char, std::allocator<char> >::const_buffers_type
-      cb8 = static_cast<const dynamic_vector_buffer<char,
+      cb10 = static_cast<const dynamic_vector_buffer<char,
         std::allocator<char> >&>(db3).data(0, 1);
-    (void)cb8;
+    (void)cb10;
 
     db1.grow(1024);
     db3.grow(1024);
@@ -446,50 +370,6 @@ void test()
   BOOST_ASIO_CHECK(n == sizeof(source_data));
   BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
 
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-  memset(dest_data, 0, sizeof(dest_data));
-  mb1 = boost::asio::buffer(dest_data);
-  mutable_buffers_1 mbc1 = boost::asio::buffer(source_data);
-  n = buffer_copy(mb1, mbc1);
-  BOOST_ASIO_CHECK(n == sizeof(source_data));
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mb1 = boost::asio::buffer(dest_data);
-  const_buffers_1 cbc1 = const_buffers_1(boost::asio::buffer(source_data));
-  n = buffer_copy(mb1, cbc1);
-  BOOST_ASIO_CHECK(n == sizeof(source_data));
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mbc1 = boost::asio::buffer(dest_data);
-  mb1 = boost::asio::buffer(source_data);
-  n = buffer_copy(mbc1, mb1);
-  BOOST_ASIO_CHECK(n == sizeof(source_data));
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mbc1 = boost::asio::buffer(dest_data);
-  cb1 = boost::asio::buffer(source_data);
-  n = buffer_copy(mbc1, cb1);
-  BOOST_ASIO_CHECK(n == sizeof(source_data));
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mbc1 = boost::asio::buffer(dest_data);
-  mutable_buffers_1 mbc2 = boost::asio::buffer(source_data);
-  n = buffer_copy(mbc1, mbc2);
-  BOOST_ASIO_CHECK(n == sizeof(source_data));
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mbc1 = boost::asio::buffer(dest_data);
-  cbc1 = const_buffers_1(boost::asio::buffer(source_data));
-  n = buffer_copy(mbc1, cbc1);
-  BOOST_ASIO_CHECK(n == sizeof(source_data));
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
   memset(dest_data, 0, sizeof(dest_data));
   mb1 = boost::asio::buffer(dest_data);
   std::vector<mutable_buffer> mv1;
@@ -542,50 +422,6 @@ void test()
   BOOST_ASIO_CHECK(n == 10);
   BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
 
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-  memset(dest_data, 0, sizeof(dest_data));
-  mb1 = boost::asio::buffer(dest_data);
-  mbc1 = boost::asio::buffer(source_data);
-  n = buffer_copy(mb1, mbc1, 10);
-  BOOST_ASIO_CHECK(n == 10);
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mb1 = boost::asio::buffer(dest_data);
-  cbc1 = const_buffers_1(boost::asio::buffer(source_data));
-  n = buffer_copy(mb1, cbc1, 10);
-  BOOST_ASIO_CHECK(n == 10);
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mbc1 = boost::asio::buffer(dest_data);
-  mb1 = boost::asio::buffer(source_data);
-  n = buffer_copy(mbc1, mb1, 10);
-  BOOST_ASIO_CHECK(n == 10);
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mbc1 = boost::asio::buffer(dest_data);
-  cb1 = boost::asio::buffer(source_data);
-  n = buffer_copy(mbc1, cb1, 10);
-  BOOST_ASIO_CHECK(n == 10);
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mbc1 = boost::asio::buffer(dest_data);
-  mbc2 = boost::asio::buffer(source_data);
-  n = buffer_copy(mbc1, mbc2, 10);
-  BOOST_ASIO_CHECK(n == 10);
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-
-  memset(dest_data, 0, sizeof(dest_data));
-  mbc1 = boost::asio::buffer(dest_data);
-  cbc1 = const_buffers_1(boost::asio::buffer(source_data));
-  n = buffer_copy(mbc1, cbc1, 10);
-  BOOST_ASIO_CHECK(n == 10);
-  BOOST_ASIO_CHECK(memcmp(dest_data, source_data, n) == 0);
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
   memset(dest_data, 0, sizeof(dest_data));
   mb1 = boost::asio::buffer(dest_data);
   mv1.clear();
@@ -662,6 +498,17 @@ struct valid_mutable_b
   mutable_buffer* end() const { return 0; }
 };
 
+struct custom_const_buffer_sequence
+{
+  operator const_buffer() const { return const_buffer(0, 0); }
+};
+
+struct custom_mutable_buffer_sequence
+{
+  operator mutable_buffer() const { return mutable_buffer(0, 0); }
+  operator const_buffer() const { return mutable_buffer(0, 0); }
+};
+
 struct invalid_const_a
 {
   typedef int value_type;
@@ -748,63 +595,65 @@ void test()
   BOOST_ASIO_CHECK(buffer_sequence_begin(b2) == &b2);
   BOOST_ASIO_CHECK(buffer_sequence_end(b2) == &b2 + 1);
 
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-  BOOST_ASIO_CHECK(is_const_buffer_sequence<const_buffers_1>::value);
-  BOOST_ASIO_CHECK(!is_mutable_buffer_sequence<const_buffers_1>::value);
-
-  const_buffers_1 b3(0, 0);
-  BOOST_ASIO_CHECK(buffer_sequence_begin(b3) == &b3);
-  BOOST_ASIO_CHECK(buffer_sequence_end(b3) == &b3 + 1);
-
-  BOOST_ASIO_CHECK(is_const_buffer_sequence<mutable_buffers_1>::value);
-  BOOST_ASIO_CHECK(is_mutable_buffer_sequence<mutable_buffers_1>::value);
-
-  mutable_buffers_1 b4(0, 0);
-  BOOST_ASIO_CHECK(buffer_sequence_begin(b4) == &b4);
-  BOOST_ASIO_CHECK(buffer_sequence_end(b4) == &b4 + 1);
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
   BOOST_ASIO_CHECK(is_const_buffer_sequence<vector<const_buffer> >::value);
   BOOST_ASIO_CHECK(!is_mutable_buffer_sequence<vector<const_buffer> >::value);
 
-  vector<const_buffer> b5;
-  BOOST_ASIO_CHECK(buffer_sequence_begin(b5) == b5.begin());
-  BOOST_ASIO_CHECK(buffer_sequence_end(b5) == b5.end());
+  vector<const_buffer> b3;
+  BOOST_ASIO_CHECK(buffer_sequence_begin(b3) == b3.begin());
+  BOOST_ASIO_CHECK(buffer_sequence_end(b3) == b3.end());
 
   BOOST_ASIO_CHECK(is_const_buffer_sequence<vector<mutable_buffer> >::value);
   BOOST_ASIO_CHECK(is_mutable_buffer_sequence<vector<mutable_buffer> >::value);
 
-  vector<mutable_buffer> b6;
-  BOOST_ASIO_CHECK(buffer_sequence_begin(b6) == b6.begin());
-  BOOST_ASIO_CHECK(buffer_sequence_end(b6) == b6.end());
+  vector<mutable_buffer> b4;
+  BOOST_ASIO_CHECK(buffer_sequence_begin(b4) == b4.begin());
+  BOOST_ASIO_CHECK(buffer_sequence_end(b4) == b4.end());
 
   BOOST_ASIO_CHECK(is_const_buffer_sequence<valid_const_a>::value);
   BOOST_ASIO_CHECK(!is_mutable_buffer_sequence<valid_const_a>::value);
 
-  valid_const_a b7;
-  BOOST_ASIO_CHECK(buffer_sequence_begin(b7) == b7.begin());
-  BOOST_ASIO_CHECK(buffer_sequence_end(b7) == b7.end());
+  valid_const_a b5;
+  BOOST_ASIO_CHECK(buffer_sequence_begin(b5) == b5.begin());
+  BOOST_ASIO_CHECK(buffer_sequence_end(b5) == b5.end());
 
   BOOST_ASIO_CHECK(is_const_buffer_sequence<valid_const_b>::value);
   BOOST_ASIO_CHECK(!is_mutable_buffer_sequence<valid_const_b>::value);
 
-  valid_const_b b8;
-  BOOST_ASIO_CHECK(buffer_sequence_begin(b8) == b8.begin());
-  BOOST_ASIO_CHECK(buffer_sequence_end(b8) == b8.end());
+  valid_const_b b6;
+  BOOST_ASIO_CHECK(buffer_sequence_begin(b6) == b6.begin());
+  BOOST_ASIO_CHECK(buffer_sequence_end(b6) == b6.end());
 
   BOOST_ASIO_CHECK(is_const_buffer_sequence<valid_mutable_a>::value);
   BOOST_ASIO_CHECK(is_mutable_buffer_sequence<valid_mutable_a>::value);
 
-  valid_mutable_a b9;
-  BOOST_ASIO_CHECK(buffer_sequence_begin(b9) == b9.begin());
-  BOOST_ASIO_CHECK(buffer_sequence_end(b9) == b9.end());
+  valid_mutable_a b7;
+  BOOST_ASIO_CHECK(buffer_sequence_begin(b7) == b7.begin());
+  BOOST_ASIO_CHECK(buffer_sequence_end(b7) == b7.end());
 
   BOOST_ASIO_CHECK(is_const_buffer_sequence<valid_mutable_b>::value);
   BOOST_ASIO_CHECK(is_mutable_buffer_sequence<valid_mutable_b>::value);
 
-  valid_mutable_b b10;
-  BOOST_ASIO_CHECK(buffer_sequence_begin(b10) == b10.begin());
-  BOOST_ASIO_CHECK(buffer_sequence_end(b10) == b10.end());
+  valid_mutable_b b8;
+  BOOST_ASIO_CHECK(buffer_sequence_begin(b8) == b8.begin());
+  BOOST_ASIO_CHECK(buffer_sequence_end(b8) == b8.end());
+
+  BOOST_ASIO_CHECK(is_const_buffer_sequence<
+      custom_const_buffer_sequence>::value);
+  BOOST_ASIO_CHECK(!is_mutable_buffer_sequence<
+      custom_const_buffer_sequence>::value);
+
+  custom_const_buffer_sequence b11;
+  BOOST_ASIO_CHECK(buffer_sequence_begin(b11) == &b11);
+  BOOST_ASIO_CHECK(buffer_sequence_end(b11) == &b11 + 1);
+
+  BOOST_ASIO_CHECK(is_const_buffer_sequence<
+      custom_mutable_buffer_sequence>::value);
+  BOOST_ASIO_CHECK(is_mutable_buffer_sequence<
+      custom_mutable_buffer_sequence>::value);
+
+  custom_mutable_buffer_sequence b12;
+  BOOST_ASIO_CHECK(buffer_sequence_begin(b12) == &b12);
+  BOOST_ASIO_CHECK(buffer_sequence_end(b12) == &b12 + 1);
 
   BOOST_ASIO_CHECK(!is_const_buffer_sequence<invalid_const_a>::value);
   BOOST_ASIO_CHECK(!is_mutable_buffer_sequence<invalid_const_a>::value);
