@@ -8,42 +8,18 @@
 #ifndef BOOST_MYSQL_TEST_COMMON_INCLUDE_TEST_COMMON_BUFFER_CONCAT_HPP
 #define BOOST_MYSQL_TEST_COMMON_INCLUDE_TEST_COMMON_BUFFER_CONCAT_HPP
 
-#include <boost/config.hpp>
 #include <boost/core/span.hpp>
 
 #include <cstdint>
-#include <cstring>
 #include <vector>
 
 namespace boost {
 namespace mysql {
 namespace test {
 
-// ARM gcc raises a spurious warning here
-#if BOOST_GCC >= 110000
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wstringop-overflow"
-#pragma GCC diagnostic ignored "-Wrestrict"
-#endif
-inline void concat(std::vector<std::uint8_t>& lhs, span<const std::uint8_t> rhs)
+inline std::vector<std::uint8_t> concat(std::vector<std::uint8_t> lhs, const std::vector<std::uint8_t>& rhs)
 {
-    if (!rhs.empty())
-    {
-        auto current_size = lhs.size();
-        lhs.resize(current_size + rhs.size());
-        std::memcpy(lhs.data() + current_size, rhs.data(), rhs.size());
-    }
-}
-#if BOOST_GCC >= 110000
-#pragma GCC diagnostic pop
-#endif
-
-inline std::vector<std::uint8_t> concat_copy(
-    std::vector<std::uint8_t> lhs,
-    const std::vector<std::uint8_t>& rhs
-)
-{
-    concat(lhs, rhs);
+    lhs.insert(lhs.end(), rhs.begin(), rhs.end());
     return lhs;
 }
 
@@ -55,7 +31,7 @@ public:
     buffer_builder() = default;
     buffer_builder& add(span<const std::uint8_t> value)
     {
-        concat(buff_, value);
+        buff_.insert(buff_.end(), value.begin(), value.end());
         return *this;
     }
     buffer_builder& add(const std::vector<std::uint8_t>& value)
